
.include "msp430x4xx.inc"
.include "lcd.inc"

SCG1 equ 0x80

.macro lcd_command(a)
  mov.b #a, r15
  call #send_command
.endm

.macro lcd_data(a)
  mov.b #a, r15
  call #send_data
.endm

  org 0x2100
start:
  ;; Turn off watchdog
  mov.w #0x5a80, &WDTCTL

  ;; Please don't interrupt me
  dint

  ;; Set up stack pointer
  mov.w #0x1900, SP

  ;; Set up crystal to 8MHz
  ;mov.b #(64|32), &DCOCTL
  ;mov.b #0xf, &BCSCTL1
  ;mov.b #0, &BCSCTL2
  mov.b #XCAP0PF, &FLL_CTL0
  bis.w #SCG1, SR
  bic.b #XT2OFF, &FLL_CTL1
osc_test:
  bic.b #OFIFG, &IFG1
  mov.b #0xff, r15
osc_pause:
  dec r15
  jnz osc_pause
  bit.b #OFIFG, &IFG1
  jnz osc_test
  bis.b #(SELM_2|SELS), &FLL_CTL1

  ;; Set up output pins
  mov.b #0x50, &P7DIR        ; P7.6=LCD_BACKLIGHT, P7.4=LCD_RESET
  mov.b #0x00, &P7OUT
  bic.b #0x50, &P7SEL        ; Not sure if this is needed
  mov.b #UCKL1|SIMO1|STE1, &P4DIR
  mov.b #STE1|UCKL1, &P4OUT
  mov.b #0xf8, &P4SEL        ; Not sure if this is needed

  ;; Set up SPI (input pin is turned off)
  mov.b #CHAR|SYNC|MM|SWRST, &U1CTL
  mov.b #CKPL|SSEL_2|STC, &U1TCTL
  mov.b #0, &U1BR1
  mov.b #4, &U1BR0                ; divide clock by 4?
  bis.b #USPIE1, &ME2
  bic.b #SWRST, &U1CTL

  ;; Okay, I can be interrupted now
  eint

init_lcd:
  ;; Reset LCD
  bic.b #0x10, &P7OUT
  call #delay
  bis.b #0x10, &P7OUT
  call #delay
  call #delay

  lcd_command(DISCTL)
  lcd_data(0)
  lcd_data(0x20)
  lcd_data(0)

  lcd_command(COMSCN)
  lcd_data(1)

  lcd_command(OSCON)
  lcd_command(SLPOUT)

  lcd_command(PWRCTR)
  lcd_data(0x0f)

  lcd_command(DISINV)
  lcd_data(1)
  lcd_data(0)
  lcd_data(2)

  lcd_command(VOLCTR)
  lcd_data(32)
  lcd_data(3)

  call #delay
  call #delay

  lcd_command(DISON)

draw_image:
  lcd_command(DATCTL)
  lcd_data(0)
  lcd_data(0)
  lcd_data(2)

  lcd_command(PASET)
  lcd_data(0)
  lcd_data(131)

  lcd_command(CASET)
  lcd_data(0)
  lcd_data(131)

  mov.w #(132*132/2)*3, r7
  mov.w #bridge_image, r8
  lcd_command(RAMWR)
repeat_draw:
  mov.b @r8+, r15
  call #send_data
  dec.w r7
  jnz repeat_draw

main:
  dec.w r9
  jnz main

  jmp main

;; send_command(r15)
send_command:
  bic.b #SIMO1, &P4OUT
  bic.b #UCKL1|SIMO1, &P4SEL   ; manually clock out bit 8 (7 to 0 is hardware)

  bic.b #STE1, &P4OUT          ; /CS=selected
  call #clock_bit
  bis.b #UCKL1|SIMO1, &P4SEL   ; use hardware SPI again

  mov.b r15, &U1TXBUF
send_command_wait_spi:
  bit.b #TXEPT, &U1TCTL
  jz send_command_wait_spi

  bis.b #STE1, &P4OUT          ; /CS=deselected
  ret

;; send_data(r15)
send_data:
  bis.b #SIMO1, &P4OUT         ; bit is 1 for data
  bic.b #UCKL1|SIMO1, &P4SEL   ; manually clock out bit 8 (7 to 0 is hardware)

  bic.b #STE1, &P4OUT          ; /CS=selected
  call #clock_bit
  bis.b #UCKL1|SIMO1, &P4SEL   ; use hardware SPI again

  mov.b r15, &U1TXBUF
send_data_wait_spi:
  bit.b #TXEPT, &U1TCTL
  jz send_command_wait_spi

  bis.b #STE1, &P4OUT          ; /CS=deselected
  ret

clock_bit:
  bic.b #UCKL1, &P4OUT        ; UCKL1=1
  nop
  nop
  nop
  nop
  bis.b #UCKL1, &P4OUT        ; UCKL2=0
  nop
  nop
  nop
  nop
  ret

delay:
  mov.w #0xffff, r15
delay_loop:
  dec r15
  jnz delay_loop
  ret

bridge_image:
  .include "bridge.inc"

  org 0xffe8
vectors:
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0 
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw start                 ; Reset


